import { XlibException } from "../exception";
/** cached query so we only get args once per js load*/
let _parsedCommandlineArgs;
/** return all key-value arguments passed tothe commandline (does not return single-value arguments)
 * example:  "myKey=myValue" will return, but "someValue" will not.
 * if you need single value args, access process.argv directly.
 */
export function getCommandlineArgs({ 
/** for better performance we cache environmental variables after the first query.   pass true to force a requery */ refresh = false } = {}) {
    if (_parsedCommandlineArgs == null || refresh === true) {
        if (typeof (process) === "undefined") {
            throw new XlibException("process (base object) is missing.  this function is meant for nodejs use.  are you running in a browser?");
        }
        else {
            //declare require: any;
            //var process = require("process");
            const rawVars = process.argv;
            _parsedCommandlineArgs = {};
            for (let i = rawVars.length - 1; i >= 0; i--) {
                const pair = rawVars[i].split("=");
                if (pair.length !== 2) {
                    //skip
                    continue;
                }
                const key = pair[0];
                const value = pair[1];
                _parsedCommandlineArgs[key] = value;
            }
        }
    }
    return _parsedCommandlineArgs;
}
/** return a key-value argument passed tothe commandline (does not return single-value arguments)
 * example:  "myKey=myValue" will return, but "someValue" will not.
 * if you need single value args, access process.argv directly.
 */
export function getCommandlineArg(key, { /**what to return if the value is null or Empty. By default, when the envVar doesn't exist we'll return undefined*/ defaultValue = undefined, 
/** for better performance we cache environmental variables after the first query.   pass true to force a requery */ refresh = false } = {}) {
    const parsedArgs = getCommandlineArgs({ refresh });
    const result = parsedArgs[key];
    if (defaultValue != null) {
        if (result == null || result.length === 0) {
            return defaultValue;
        }
    }
    return result;
}
//# sourceMappingURL=node-helper.js.map