/** a Map extended to allow keys to expire a given time after setting. */
export declare class ExpiresMap<K, V> extends Map<K, V> {
    /** how long in Ms before an entity expires */
    expireMs: Numeric;
    /** tracks expirations */
    private _handleMap;
    constructor(
    /** how long in Ms before an entity expires */
    expireMs: Numeric, 
    /** optionally allows pre-populating the map. these are still subject to expiration   */
    entries?: Iterable<[K, V]>);
    set(key: K, value: V, customExpireMs?: Numeric): this;
    delete(key: K): boolean;
}
/** up to 32 true/false values stored in 32bits (a bitmask) */
export declare class BitFlags {
    /** how many bytes can be stored by instances of this */
    static MAXFLAGS: number;
    /** all flags set to true (ctor of numHelper.INT32_MAX) */
    static ALL: BitFlags;
    /** all flags set to false (ctor of 0) */
    static NONE: BitFlags;
    /** internal storage for our bitflags (just a number!) */
    rawBuffer: number;
    constructor(bitFlagsOrRawBuffer?: BitFlags | number);
    private static _assertIndexInRange;
    /** return the value of a certain flag */
    getFlag(index: number): boolean;
    static getFlag(rawBuffer: number, index: number): boolean;
    setFlag(index: number, value: boolean): void;
    static setFlag(rawBuffer: number, index: number, value: boolean): void;
    /** flips the value of the flag */
    flipFlag(index: number): void;
    static flipFlag(rawBuffer: number, index: number): void;
    /** reset everything to false */
    clear(): void;
    /** sets any flags that are set in the input */
    add(bitFlagsOrRawBuffer: BitFlags | number): void;
    static add(rawBuffer: number, bitFlagsOrRawBuffer: BitFlags | number): void;
    /** unsets any set flags that are set in the input */
    subtract(bitFlagsOrRawBuffer: BitFlags | number): void;
    /** unsets any set flags that are set in the input */
    static subtract(rawBuffer: number, bitFlagsOrRawBuffer: BitFlags | number): void;
    /** flips the value of any flags set in the input */
    flip(bitFlagsOrRawBuffer: BitFlags | number): void;
    /** flips the value of any flags set in the input */
    static flip(rawBuffer: number, bitFlagsOrRawBuffer: BitFlags | number): void;
    /** returns true if all the set flags in the input are also set in this. */
    isAllOn(bitFlagsOrRawBuffer: BitFlags | number): boolean;
    /** returns true if all the set flags in the input are also set in this. */
    static isAllOn(rawBuffer: number, bitFlagsOrRawBuffer: BitFlags | number): boolean;
    /** returns true if any of the set flags in the input are set in this. */
    isAnyOn(bitFlagsOrRawBuffer: BitFlags | number): boolean;
    /** returns true if any of the set flags in the input are set in this. */
    static isAnyOn(rawBuffer: number, bitFlagsOrRawBuffer: BitFlags | number): boolean;
    /** returns true if the set and unset flags exactly match */
    equals(bitFlagsOrRawBuffer: BitFlags | number): boolean;
    /** returns true if the set and unset flags exactly match */
    static equals(rawBuffer: number, bitFlagsOrRawBuffer: BitFlags | number): boolean;
    getHashCode(): number;
}
/**
 *  enumerate over the key+items in a collection, removing each pair as they are enumerated.
 *  return a rejected promise from the callback to abort enumeration.  item is removed from collection immediatly prior to the callback being invoked, so if you wish it to remain in the collection you will need to manually re-add it.
 * @param collection
 * @param callback
 */
export declare function forEachAndRemove<TItem>(collection: {
    [key: string]: TItem;
}, callback: (item: TItem, key: string) => PromiseLike<ANY>): Promise<void>;
