/** get the consistent `globalThis` object
 * @remarks
 * pollyfill found in "./internal/pollyfill.ts"
 */
export declare function getGlobal(): typeof globalThis;
/** reads specific envVar keys `NODE_ENV` or `ENV` for values that specify an environment.
 * @remarks
 * see {@link getEnvVar}
    */
export declare type EnvLevel = "dev" | "test" | "uat" | "prod";
/** read specific enVar key `LOG_LEVEL`  used by the xlib.diagnostics.logging system to determine what messages to filter out */
export declare type LogLevel = "trace" | 
/** Exception details default to include a full stack trace */
"debug" | 
/** Exception details default to include the first frame of the stack */
"info" | 
/** Exception details default to include no stack */
"warn" | "error" | "fatal" | "silent";
export declare type PlatformType = "browser" | "node" | "embedded" | "unknown";
export declare type OsName = "win" | "linux" | "mac" | "ios" | "android" | "unknown";
export interface IEnvInfo {
    platform: PlatformType;
    os: OsName;
    /** defaults to {@link EnvLevel.PROD} unless otherwise set via envVars.  */
    env: EnvLevel;
    log: LogLevel;
    rawDetails: {
        /** platform details according to https://github.com/bestiejs/platform.js/ */
        platformJs: {
            description?: string;
            /**
             * The name of the browser's layout engine.
             *
             * The list of common layout engines include:
             * "Blink", "EdgeHTML", "Gecko", "Trident" and "WebKit"
             */
            layout?: string;
            /**
             * The name of the product's manufacturer.
             *
             * The list of manufacturers include:
             * "Apple", "Archos", "Amazon", "Asus", "Barnes & Noble", "BlackBerry",
             * "Google", "HP", "HTC", "LG", "Microsoft", "Motorola", "Nintendo",
             * "Nokia", "Samsung" and "Sony"
             */
            manufacturer?: string;
            /**
             * The name of the browser/environment.
             *
             * The list of common browser names include:
             * "Chrome", "Electron", "Firefox", "Firefox for iOS", "IE",
             * "Microsoft Edge", "PhantomJS", "Safari", "SeaMonkey", "Silk",
             * "Opera Mini" and "Opera"
             *
             * Mobile versions of some browsers have "Mobile" appended to their name:
             * eg. "Chrome Mobile", "Firefox Mobile", "IE Mobile" and "Opera Mobile"
             */
            name?: string;
            /**
             * The alpha/beta release indicator.
             */
            prerelease?: string;
            /**
             * The name of the product hosting the browser.
             *
             * The list of common products include:
             *
             * "BlackBerry", "Galaxy S4", "Lumia", "iPad", "iPod", "iPhone", "Kindle",
             * "Kindle Fire", "Nexus", "Nook", "PlayBook", "TouchPad" and "Transformer"
             */
            product?: string;
            /**
             * The browser's user agent string.
             */
            ua?: string;
            /**
             * The version of the OS.
             */
            version?: string;
            /**
             * The name of the operating system.
             */
            os?: {
                /**
                 * The CPU architecture the OS is built for.
                 */
                architecture?: number;
                /**
                 * The family of the OS.
                 *
                 * Common values include:
                 * "Windows", "Windows Server 2008 R2 / 7", "Windows Server 2008 / Vista",
                 * "Windows XP", "OS X", "Linux", "Ubuntu", "Debian", "Fedora", "Red Hat",
                 * "SuSE", "Android", "iOS" and "Windows Phone"
                 */
                family?: string;
                /**
                 * The version of the OS.
                 */
                version?: string;
            };
        };
    };
}
export declare function getEnvInfo({ 
/** by default returns cached result of last call.  pass true to requery envInfo  */
refresh }?: {
    refresh?: boolean | undefined;
}): IEnvInfo;
export declare function getEnvLevel(): EnvLevel;
/** returns true if "prod" envLevel */
export declare function isProd(): boolean;
/** returns true if "trace" || "debug" logLevel. */
export declare function isDebug(): boolean;
export declare function getLogLevel(): LogLevel;
/** read an "environment variable".
nodejs / phantomjs: reads from commandline switches (prefered) or system environment variables.
browser:  reads from querystring (prefered), or cookie, or <html data-key> attribute (least prefered).
 */
export declare function getEnvironmentVariable(key: string, { /**what to return if the value is null or Empty. By default, when the envVar doesn't exist we'll return undefined*/ defaultValue, 
/** for better performance we cache environmental variables after the first query.   pass true to force a requery */ refresh }?: {
    defaultValue?: string | undefined;
    refresh?: boolean | undefined;
}): string | undefined;
