import * as browserHelper from "./_internal/browser-helper";
import * as nodeHelper from "./_internal/node-helper";
import { XlibException } from "./exception";
import * as _ from "lodash";
//import { parseEnum } from "./util/enum-helper"
import * as platformJs from "platform";
/** get the consistent `globalThis` object
 * @remarks
 * pollyfill found in "./internal/pollyfill.ts"
 */
export function getGlobal() {
    return globalThis;
}
let _envInfo;
export function getEnvInfo({ 
/** by default returns cached result of last call.  pass true to requery envInfo  */
refresh = false } = {}) {
    var _a, _b;
    if (_envInfo == null || refresh !== true) {
        _envInfo = {
            platform: (() => {
                var _a, _b;
                const name = (_a = platformJs.name) === null || _a === void 0 ? void 0 : _a.toLowerCase();
                if (name == null)
                    return "unknown";
                if (name.includes("browser"))
                    return "browser";
                if (name.includes("mobile"))
                    return "browser";
                if (name.includes("node"))
                    return "node";
                switch (name.slice(0, 4)) {
                    case "chro":
                    case "fire":
                    case "ie":
                    case "ie m": //"ie mobile"
                    case "edge":
                    case "safa":
                    case "oper":
                    case "maxt": //maxthon
                        return "browser";
                    case "phan": //phantomjs
                    case "adob": //adobe air
                    case "rhin": //rhino
                    case "ring": //ringojs
                    case "elec": //electron
                    case "nw.j": //nw.js
                        return "embedded";
                }
                if (name.includes(" os"))
                    return "embedded";
                const desc = (_b = platformJs.description) === null || _b === void 0 ? void 0 : _b.toLowerCase();
                if (desc == null)
                    return "unknown";
                if (desc.includes("browser"))
                    return "browser";
                if (desc.includes("firefox"))
                    return "browser";
                if (desc.includes("chrom"))
                    return "browser";
                return "unknown";
            })(),
            os: (() => {
                if (typeof (process) !== "undefined") {
                    switch (process.platform) {
                        case "win32":
                            return "win";
                        case "darwin":
                            return "mac";
                        case "linux":
                            return "linux";
                        default:
                            return "unknown";
                    }
                }
                try {
                    if (navigator.appVersion.indexOf("Win") !== -1) {
                        return "win";
                    }
                    if (navigator.appVersion.indexOf("Mac") !== -1) {
                        return "mac";
                    }
                    if (navigator.appVersion.indexOf("Linux") !== -1) {
                        return "linux";
                    }
                }
                catch (ex) { }
                return "unknown";
            })(),
            env: "dev",
            log: "trace",
            rawDetails: {
                platformJs: _.cloneDeep(platformJs),
            }
        };
        //last minute fixup detections:  platform.js sucks, doesn't set platform properly quite often.  fix it up for browsers and node
        {
            //force node if node explicit values found, assuming platform.js didn't detect some fancy embedded
            if (_envInfo.platform !== "embedded") {
                if (typeof process !== "undefined" && /node|io\.js/.test((_a = process === null || process === void 0 ? void 0 : process.release) === null || _a === void 0 ? void 0 : _a.name)) {
                    //detect node, from: https://www.geeksforgeeks.org/how-to-check-whether-a-script-is-running-under-node-js-or-not/				
                    _envInfo.platform = "node";
                }
            }
            if (_envInfo.platform === "unknown") {
                if (typeof (window) !== "undefined") {
                    _envInfo.platform = "browser";
                }
            }
            if (((_b = platformJs.os) === null || _b === void 0 ? void 0 : _b.family) != null) {
                const fam = platformJs.os.family.toLowerCase();
                if (fam.includes("android"))
                    _envInfo.os = "android";
                if (fam.startsWith("ios"))
                    _envInfo.os = "ios";
                if (_envInfo.os === "unknown") {
                    _envInfo.os = (() => {
                        switch (fam.slice(0, 3)) {
                            case "win": return "win";
                            case "mac": return "mac";
                            case "and": return "android";
                            case "ios": return "ios";
                            case "lin":
                            case "ubu":
                            case "kai":
                            case "cen":
                            case "deb":
                            case "fed":
                            case "kub":
                                return "linux";
                            default:
                                return "unknown";
                        }
                    })();
                }
            }
        }
        //set env details
        _envInfo.env = (() => {
            var _a, _b;
            const foundEnvVar = (_b = (_a = getEnvironmentVariable("NODE_ENV")) !== null && _a !== void 0 ? _a : getEnvironmentVariable("ENV")) !== null && _b !== void 0 ? _b : "DEV";
            switch (foundEnvVar === null || foundEnvVar === void 0 ? void 0 : foundEnvVar.trim().slice(0, 3).toLowerCase()) {
                case "dev":
                    return "dev";
                case "tes":
                    return "test";
                case "uat":
                    return "uat";
                case "pro":
                    return "prod";
                default:
                    console.warn(`unknown envLevel found in "NODE_ENV" | "ENV" envVars.  found "${foundEnvVar}".  defaulting to DEV`);
                    return "dev";
            }
        })();
        //_envInfo.rawDetails.initialEnv = _envInfo.env
        //set log details
        _envInfo.log = (() => {
            const foundEnvVar = getEnvironmentVariable("LOG_LEVEL");
            switch (foundEnvVar === null || foundEnvVar === void 0 ? void 0 : foundEnvVar.trim().slice(0, 3).toLowerCase()) {
                case "tra":
                    return "trace";
                case "deb":
                    return "debug";
                case "inf":
                    return "info";
                case "war":
                    return "warn";
                case "err":
                    return "error";
                case "fat":
                    return "fatal";
                case "sil":
                    return "silent";
                default:
                    console.warn(`unknown logLevel found in "LOG_LEVEL" envVar.  found "${foundEnvVar}".  defaulting to TRACE`);
                    return "trace";
            }
            // const toReturn = parseEnum( LogLevel, foundEnvVar )
            // if ( toReturn == null ) {
            // 	console.warn( `unknown logLevel found in "LOG_LEVEL" envVar.  found "${ foundEnvVar }".  defaulting to TRACE` )
            // 	return LogLevel.TRACE
            // }
            // return toReturn
            // // LogLevel[ foundEnvVar?.toUpperCase() as never ]
            // if ( typeof toReturn === "string" ) return LogLevel[ toReturn as never ] as never as LogLevel
            // if ( typeof toReturn === "number" ) return toReturn as never as LogLevel
            // console.warn( `unknown logLevel found in envVars.  found "${ foundEnvVar }".  defaulting to TRACE` )
            // return LogLevel.TRACE
        })();
    }
    // //disallow external modification
    // const toReturn = _.clone( _envInfo )
    // //apply overrides
    // if ( _envLevelOverride !== undefined ) {
    // 	toReturn.env = _envLevelOverride
    // }
    return _envInfo;
}
// interface IEnum<T> {
// 	[ id: string ]: IEnum<T>[keyof IEnum<T>];
// 	[ nu: number ]: IEnum<T>[keyof IEnum<T>];
// }
// let _envLevelOverride: EnvLevel | undefined
// /**  Override the envLevel.  You can manually change this at runtime, but if so, it is advisable to do so only once, at the start of your application */
// export function setEnvLevel( env: EnvLevel ) {
// 	_envLevelOverride = env
// }
export function getEnvLevel() {
    var _a;
    //	return _envLevelOverride ?? _envInfo?.env ?? getEnvInfo().env
    return (_a = _envInfo === null || _envInfo === void 0 ? void 0 : _envInfo.env) !== null && _a !== void 0 ? _a : getEnvInfo().env;
}
/** returns true if "prod" envLevel */
export function isProd() {
    return getEnvLevel() === "prod";
}
/** returns true if "trace" || "debug" logLevel. */
export function isDebug() {
    return _.includes(["trace", "debug"], getLogLevel());
}
export function getLogLevel() {
    var _a;
    return (_a = _envInfo === null || _envInfo === void 0 ? void 0 : _envInfo.log) !== null && _a !== void 0 ? _a : getEnvInfo().log;
}
/** read an "environment variable".
nodejs / phantomjs: reads from commandline switches (prefered) or system environment variables.
browser:  reads from querystring (prefered), or cookie, or <html data-key> attribute (least prefered).
 */
export function getEnvironmentVariable(key, { /**what to return if the value is null or Empty. By default, when the envVar doesn't exist we'll return undefined*/ defaultValue = undefined, 
/** for better performance we cache environmental variables after the first query.   pass true to force a requery */ refresh = false } = {}) {
    let result;
    switch (_envInfo.platform) {
        case "browser":
            //try to find in querystring
            result = browserHelper.getQuerystringVariable(key);
            if (result != null && result.length > 0) {
                break;
            }
            //try to find in cookie
            result = browserHelper.getCookie(key);
            if (result != null && result.length > 0) {
                break;
            }
            //try to find in html attribute
            result = browserHelper.getDomAttribute("html", "data-" + key, true);
            if (result != null && result.length > 0) {
                break;
            }
            result = browserHelper.getDomAttribute("html", key, true);
            if (result != null && result.length > 0) {
                break;
            }
            break;
        case "node":
        case "embedded":
            //seach commandline
            result = nodeHelper.getCommandlineArg(key);
            if (result != null && result.length > 0) {
                break;
            }
            //search system env vars
            result = process.env[key];
            if (result != null && result.length > 0) {
                break;
            }
            break;
        default:
            throw new XlibException(`unsupported plaform type.  platform=${_envInfo.platform}  envInfo=${JSON.stringify(_envInfo)}`);
    }
    //have our result (including if it's null/empty)
    if (defaultValue == null) {
        return result;
    }
    if (result == null || result.length === 0) {
        return defaultValue;
    }
    return result;
}
//# sourceMappingURL=environment.js.map