"use strict";
/* eslint-disable no-bitwise */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.forEachAndRemove = exports.BitFlags = exports.ExpiresMap = void 0;
const exception = __importStar(require("./exception"));
const num = __importStar(require("./num"));
//import numHelper = require( "./_util/numhelper" );
//import diag = require( "./diagnostics" );
/** a Map extended to allow keys to expire a given time after setting. */
class ExpiresMap extends Map {
    constructor(
    /** how long in Ms before an entity expires */
    expireMs, 
    /** optionally allows pre-populating the map. these are still subject to expiration   */
    entries) {
        super();
        this.expireMs = expireMs;
        /** tracks expirations */
        this._handleMap = new Map();
        if (entries != null) {
            for (const [key, val] of entries) {
                this.set(key, val);
            }
        }
    }
    set(key, value, customExpireMs) {
        customExpireMs = customExpireMs || this.expireMs;
        //remove existing handle, if any
        {
            const handle = this._handleMap.get(key);
            if (handle != null) {
                clearTimeout(handle);
            }
        }
        super.set(key, value);
        //set new handle
        {
            const handle = setTimeout(() => {
                clearTimeout(handle);
                this._handleMap.delete(key);
                this.delete(key);
            }, customExpireMs.valueOf());
            this._handleMap.set(key, handle);
        }
        return this;
    }
    delete(key) {
        //remove existing handle, if any
        {
            const handle = this._handleMap.get(key);
            if (handle != null) {
                clearTimeout(handle);
            }
            this._handleMap.delete(key);
        }
        return super.delete(key);
    }
}
exports.ExpiresMap = ExpiresMap;
/** up to 32 true/false values stored in 32bits (a bitmask) */
class BitFlags {
    constructor(bitFlagsOrRawBuffer) {
        /** internal storage for our bitflags (just a number!) */
        this.rawBuffer = 0;
        if (bitFlagsOrRawBuffer == null) {
            this.rawBuffer = 0;
        }
        else if (typeof (bitFlagsOrRawBuffer) === "number") {
            const rawBufferInput = bitFlagsOrRawBuffer;
            this.rawBuffer = rawBufferInput;
        }
        else {
            const flags = bitFlagsOrRawBuffer;
            this.rawBuffer = flags.rawBuffer;
        }
    }
    static _assertIndexInRange(index) {
        if (index < BitFlags.MAXFLAGS && index >= 0) {
            return;
        }
        throw new exception.XlibException(`index out of bounds.  You supplied ${index} while expected range is 0 to ${BitFlags.MAXFLAGS - 1}`);
    }
    /** return the value of a certain flag */
    getFlag(index) {
        BitFlags._assertIndexInRange(index);
        return ((this.rawBuffer & (0x01 << index)) !== 0);
    }
    static getFlag(rawBuffer, index) {
        BitFlags._assertIndexInRange(index);
        return ((rawBuffer & (0x01 << index)) !== 0);
    }
    setFlag(index, value) {
        BitFlags._assertIndexInRange(index);
        if (value) {
            this.rawBuffer |= (0x01) << index;
        }
        else {
            this.rawBuffer &= (~(0x01) << index);
        }
    }
    static setFlag(rawBuffer, index, value) {
        BitFlags._assertIndexInRange(index);
        if (value) {
            rawBuffer |= (0x01) << index;
        }
        else {
            rawBuffer &= (~(0x01) << index);
        }
    }
    /** flips the value of the flag */
    flipFlag(index) {
        BitFlags._assertIndexInRange(index);
        this.rawBuffer ^= ((0x01 << index));
    }
    static flipFlag(rawBuffer, index) {
        BitFlags._assertIndexInRange(index);
        rawBuffer ^= ((0x01 << index));
    }
    /** reset everything to false */
    clear() {
        this.rawBuffer = 0;
    }
    /** sets any flags that are set in the input */
    add(bitFlagsOrRawBuffer) {
        if (typeof (bitFlagsOrRawBuffer) === "number") {
            const rawBufferInput = bitFlagsOrRawBuffer;
            this.rawBuffer |= rawBufferInput;
        }
        else {
            const flags = bitFlagsOrRawBuffer;
            this.rawBuffer |= flags.rawBuffer;
        }
    }
    static add(rawBuffer, bitFlagsOrRawBuffer) {
        if (typeof (bitFlagsOrRawBuffer) === "number") {
            const rawBufferInput = bitFlagsOrRawBuffer;
            rawBuffer |= rawBufferInput;
        }
        else {
            const flags = bitFlagsOrRawBuffer;
            rawBuffer |= flags.rawBuffer;
        }
    }
    /** unsets any set flags that are set in the input */
    subtract(bitFlagsOrRawBuffer) {
        if (typeof (bitFlagsOrRawBuffer) === "number") {
            const rawBufferInput = bitFlagsOrRawBuffer;
            this.rawBuffer &= ~rawBufferInput;
        }
        else {
            const flags = bitFlagsOrRawBuffer;
            this.rawBuffer &= ~flags.rawBuffer;
        }
    }
    /** unsets any set flags that are set in the input */
    static subtract(rawBuffer, bitFlagsOrRawBuffer) {
        if (typeof (bitFlagsOrRawBuffer) === "number") {
            const rawBufferInput = bitFlagsOrRawBuffer;
            rawBuffer &= ~rawBufferInput;
        }
        else {
            const flags = bitFlagsOrRawBuffer;
            rawBuffer &= ~flags.rawBuffer;
        }
    }
    /** flips the value of any flags set in the input */
    flip(bitFlagsOrRawBuffer) {
        if (typeof (bitFlagsOrRawBuffer) === "number") {
            const rawBufferInput = bitFlagsOrRawBuffer;
            this.rawBuffer ^= rawBufferInput;
        }
        else {
            const flags = bitFlagsOrRawBuffer;
            this.rawBuffer ^= flags.rawBuffer;
        }
    }
    /** flips the value of any flags set in the input */
    static flip(rawBuffer, bitFlagsOrRawBuffer) {
        if (typeof (bitFlagsOrRawBuffer) === "number") {
            const rawBufferInput = bitFlagsOrRawBuffer;
            rawBuffer ^= rawBufferInput;
        }
        else {
            const flags = bitFlagsOrRawBuffer;
            rawBuffer ^= flags.rawBuffer;
        }
    }
    /** returns true if all the set flags in the input are also set in this. */
    isAllOn(bitFlagsOrRawBuffer) {
        if (typeof (bitFlagsOrRawBuffer) === "number") {
            const rawBufferInput = bitFlagsOrRawBuffer;
            return (this.rawBuffer & rawBufferInput) === rawBufferInput;
        }
        else {
            const flags = bitFlagsOrRawBuffer;
            return (this.rawBuffer & flags.rawBuffer) === flags.rawBuffer;
        }
    }
    /** returns true if all the set flags in the input are also set in this. */
    static isAllOn(rawBuffer, bitFlagsOrRawBuffer) {
        if (typeof (bitFlagsOrRawBuffer) === "number") {
            const rawBufferInput = bitFlagsOrRawBuffer;
            return (rawBuffer & rawBufferInput) === rawBufferInput;
        }
        else {
            const flags = bitFlagsOrRawBuffer;
            return (rawBuffer & flags.rawBuffer) === flags.rawBuffer;
        }
    }
    /** returns true if any of the set flags in the input are set in this. */
    isAnyOn(bitFlagsOrRawBuffer) {
        if (typeof (bitFlagsOrRawBuffer) === "number") {
            const rawBufferInput = bitFlagsOrRawBuffer;
            return (this.rawBuffer & rawBufferInput) !== 0;
        }
        else {
            const flags = bitFlagsOrRawBuffer;
            return (this.rawBuffer & flags.rawBuffer) !== 0;
        }
    }
    /** returns true if any of the set flags in the input are set in this. */
    static isAnyOn(rawBuffer, bitFlagsOrRawBuffer) {
        if (typeof (bitFlagsOrRawBuffer) === "number") {
            const rawBufferInput = bitFlagsOrRawBuffer;
            return (rawBuffer & rawBufferInput) !== 0;
        }
        else {
            const flags = bitFlagsOrRawBuffer;
            return (rawBuffer & flags.rawBuffer) !== 0;
        }
    }
    /** returns true if the set and unset flags exactly match */
    equals(bitFlagsOrRawBuffer) {
        if (typeof (bitFlagsOrRawBuffer) === "number") {
            const rawBufferInput = bitFlagsOrRawBuffer;
            return this.rawBuffer === rawBufferInput;
        }
        else {
            const flags = bitFlagsOrRawBuffer;
            return this.rawBuffer === flags.rawBuffer;
        }
    }
    /** returns true if the set and unset flags exactly match */
    static equals(rawBuffer, bitFlagsOrRawBuffer) {
        if (typeof (bitFlagsOrRawBuffer) === "number") {
            const rawBufferInput = bitFlagsOrRawBuffer;
            return rawBuffer === rawBufferInput;
        }
        else {
            const flags = bitFlagsOrRawBuffer;
            return rawBuffer === flags.rawBuffer;
        }
    }
    getHashCode() {
        return this.rawBuffer;
    }
}
exports.BitFlags = BitFlags;
/** how many bytes can be stored by instances of this */
BitFlags.MAXFLAGS = 32;
//public static SIZEOF = 32;
/** all flags set to true (ctor of numHelper.INT32_MAX) */
BitFlags.ALL = new BitFlags(num.INT32_MAX);
/** all flags set to false (ctor of 0) */
BitFlags.NONE = new BitFlags(0);
/**
 *  enumerate over the key+items in a collection, removing each pair as they are enumerated.
 *  return a rejected promise from the callback to abort enumeration.  item is removed from collection immediatly prior to the callback being invoked, so if you wish it to remain in the collection you will need to manually re-add it.
 * @param collection
 * @param callback
 */
async function forEachAndRemove(collection, callback) {
    const keys = Object.keys(collection);
    let nextIndex = 0;
    const toReturn = new Promise((resolve, reject) => {
        function _iterationWorker() {
            if (nextIndex >= keys.length) {
                resolve();
                return;
            }
            const currentIndex = nextIndex;
            nextIndex++;
            const currentKey = keys[currentIndex];
            const currentItem = collection[currentKey];
            //remove value to be enumerated
            delete collection[currentKey];
            callback(currentItem, currentKey)
                .then(() => {
                _iterationWorker();
            }, (err) => {
                reject(err);
                return undefined;
            });
        }
        _iterationWorker();
    });
    return toReturn;
}
exports.forEachAndRemove = forEachAndRemove;
//# sourceMappingURL=collection.js.map