"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Autoscaler = exports.ReadWriteLock = exports.CreateExposedPromise = exports.createMutablePromise = exports.exposeStatus = exports.awaitInspect = exports.retry = exports.RetryStopError = exports.TimeoutRejectError = exports.timeout = exports.delay = exports.rx = void 0;
const exception = __importStar(require("./exception"));
const _ = __importStar(require("lodash"));
const diag = __importStar(require("./diagnostics"));
const rx = __importStar(require("rxjs"));
exports.rx = rx;
/** a promise that resolves after an amount of time.   good for delaying execution
 */
function delay(ms) {
    return new Promise((resolve, reject) => {
        setTimeout(() => resolve(), ms.valueOf());
    });
}
exports.delay = delay;
/** returns a new promise that will reject with a {@link TimeoutRejectError} if the initial promise does not resolve in the required timeframe
 * Important:  if a timeout occurs, the initial promise is not aborted.  It may later resolve successfully even though it won't be listened to.
 */
function timeout(ms, initial, rejectMessage = `xlib.promise.timeout(${ms.valueOf()}) exceeded`) {
    return Promise.race([initial, delay(ms.valueOf()).then(() => { return Promise.reject(new TimeoutRejectError(rejectMessage)); })]);
}
exports.timeout = timeout;
/** rejection raised by {@ timeout} */
class TimeoutRejectError extends exception.XlibException {
}
exports.TimeoutRejectError = TimeoutRejectError;
/** error that is thrown by the [[retry]] method if number of retries is exceeded.  You may also manually reject with this to abort execution of the retry() logic */
class RetryStopError extends exception.XlibException {
}
exports.RetryStopError = RetryStopError;
/**
 * retries a failed promise untill success or exit condition occurs.
 * to abort early, throw or reject with an { @link xlib.promise.RetryStopError }
 * @param options
 * @param fn
 */
async function retry(options, fn) {
    options = Object.assign({ 
        //default implementation
        acceptResult: ({ isSuccess, err, result, promise }) => { return isSuccess === true; }, 
        //maxRetries: 1,
        minDelay: 0, maxDelay: 100, expGrowDelay: 1.5, maxJitter: 100 }, options); //as Required<IRetryOptions>
    ///
    let attempts = 0;
    let lastDelayMs = 0;
    /** used if the options.throw_original is set to true */
    let firstRejectedResult;
    /** decide if should retry, and if should delay, does so */
    async function shouldRetry() {
        if (attempts > options.maxRetries) {
            if (options.throw_original === true) {
                return Promise.reject(firstRejectedResult);
            }
            return Promise.reject(new RetryStopError(`too many attempts.   maxRetries=${options.maxRetries}  attempts=${attempts}`));
        }
        //calc delay amount
        let delayMs = Math.max(options.minDelay, lastDelayMs);
        delayMs = Math.pow(delayMs, options.expGrowDelay);
        //clamp within min/max		
        delayMs = Math.max(options.minDelay, delayMs);
        delayMs = Math.min(delayMs, options.maxDelay);
        //apply jitter 
        const jitter = Math.floor((options.maxJitter * (Math.random() - 0.5)) * 2);
        delayMs += jitter;
        //clamp again
        delayMs = Math.max(options.minDelay, delayMs);
        delayMs = Math.min(delayMs, options.maxDelay);
        delayMs = Math.floor(delayMs);
        //do the delay
        //console.log( `${ attempts } about to delay: ${ delayMs }, ${ JSON.stringify( { ...options, jitter } ) }` )
        await delay(delayMs);
        // eslint-disable-next-line require-atomic-updates
        lastDelayMs = delayMs;
    }
    //retry loop
    while (true) {
        await shouldRetry();
        //let shouldContinue = 
        // if ( !shouldContinue ) {
        // 	break;
        // }
        attempts++;
        const promise = fn();
        try {
            const result = await promise;
            if (options.acceptResult({ isSuccess: true, result, promise })) {
                return promise;
            }
            else {
                //try again
                firstRejectedResult = firstRejectedResult !== null && firstRejectedResult !== void 0 ? firstRejectedResult : result;
                continue;
            }
        }
        catch (err) {
            if (err instanceof RetryStopError) {
                //if caller rejects with a stopError, abort
                return Promise.reject(err);
            }
            if (options.acceptResult({ isSuccess: false, err, promise })) {
                return promise;
            }
            else {
                //try again
                firstRejectedResult = firstRejectedResult !== null && firstRejectedResult !== void 0 ? firstRejectedResult : err;
                continue;
            }
        }
    }
    //too many attempts
    throw new exception.XlibException("should not get here");
}
exports.retry = retry;
/**
 * @deprecated
 * use {@link exposeStatus} instead
 * @obsolete
 * */
exports.awaitInspect = exposeStatus;
/** adds a ```.status()``` property to your existing promise, allowing inspection of it's current state.   If the promise already has a ```status``` property, an XlibException is thrown. */
function exposeStatus(promise) {
    const toReturn = promise;
    if (toReturn.status != null) {
        throw new exception.XlibException("can not expose promise status because the given promise already has a .status property");
    }
    const status = {
        isPending: true
    };
    toReturn.then((result) => {
        status.isPending = false;
        status.isSuccess = true;
        status.result = result;
        return result;
    }, (error) => {
        status.isPending = false;
        status.isSuccess = false;
        status.error = error;
        return Promise.reject(error);
    });
    toReturn.status = () => {
        return Object.assign({}, status);
    };
    return toReturn;
}
exports.exposeStatus = exposeStatus;
/** create a promise who's resolve/reject occur from an outside system.
 * @example
 * //for example, resolve the promise by calling
 * let myProm = createMutablePromise();
 * myProm.resolve();
 *  */
function createMutablePromise() {
    /** because normal promise resolve() may occur on the next tick, createMutablePromise needs to implement it's own status workflow to be instantly available. */
    const status = {
        isPending: true
    };
    let _resolve = null;
    let _reject = null;
    const toReturn = new Promise((resolve, reject) => {
        _resolve = async (result) => {
            status.isPending = false;
            status.isSuccess = true;
            status.result = result;
            return resolve(result);
        };
        _reject = (reason) => {
            status.isPending = false;
            status.isSuccess = false;
            status.error = reason;
            return reject(reason);
        };
    });
    toReturn.resolve = _resolve;
    toReturn.reject = _reject;
    toReturn.status = () => { return Object.assign({}, status); };
    return toReturn;
}
exports.createMutablePromise = createMutablePromise;
/** @deprecated use [[createMutablePromise]] instead */
exports.CreateExposedPromise = createMutablePromise;
/** Synchronization construct.  Allows for exclusive writes and parallel reads.  requests are processed in a FIFO order.
 *
 * Usage notes:
 * - You should hold a lock for as little time as possible to avoid blocking other users.
 * - Be sure to call .[[readEnd]] or .[[writeEnd]] to avoid deadlocks
 * @example
 * const lock = new ReadWriteLock("start")
 * try{
 * 	const initialValue = lock.writeBegin()
 * 	console.log("initialValue==='start'",initialValue==="start")
 * }finally{
 * 	lock.writeEnd("finish")
 * }
 *
 */
class ReadWriteLock {
    /**
     *
     * @param value optional data to synchronize access to
     */
    constructor(value) {
        /** multiple reads can occur at the same time */
        this._currentReads = [];
        /** requests are processed FIFO.  pending requests are stored here. */
        this._pendingTransactions = [];
        this._value = value;
    }
    _tryProcessNext() {
        var _a, _b;
        if (((_a = this._currentWrite) === null || _a === void 0 ? void 0 : _a.status().isPending) === true)
            return;
        if (this._currentReads.length > 0 && ((_b = this._pendingTransactions[0]) === null || _b === void 0 ? void 0 : _b.isRead) === false)
            return;
        const nextTrans = this._pendingTransactions.shift();
        void (nextTrans === null || nextTrans === void 0 ? void 0 : nextTrans.promise.then(() => { this._tryProcessNext(); }));
        nextTrans === null || nextTrans === void 0 ? void 0 : nextTrans.promise.resolve();
    }
    async read(readFcn) {
        // return Promise.resolve<TReturn>(
        // 	this.readBegin().then( ( val ) => {
        // 	return readFcn( val )
        // } )
        // ).finally( () => { this.readEnd() } )
        const val = await this.readBegin();
        const toReturn = await readFcn(val);
        return this.readEnd();
        //return toReturn
    }
    async write(writeFcn) {
        //console.log( "write:start" )
        const val = await this.writeBegin();
        //console.log( "write:writeBegin()->", val )
        const newVal = await writeFcn(val);
        //console.log( "write:writeFcn()->", newVal )
        return this.writeEnd(newVal);
    }
    tryReadBegin() {
        if (this._currentWrite != null || this._pendingTransactions.length > 0) {
            return { lockAquired: false };
        }
        this._currentReads.push(createMutablePromise());
        return { lockAquired: true, value: this._value };
    }
    async readBegin() {
        const fastTry = this.tryReadBegin();
        if (fastTry.lockAquired === true) {
            return fastTry.value;
        }
        try {
            //enquue our read in a FIFO order
            //if ( this._pendingTransactions.length > 0 ) {
            const readTransaction = createMutablePromise();
            const toReturn = new Promise((resolve, reject) => {
                void readTransaction.then(() => {
                    diag.throwCheck(this._currentWrite == null, "race, current write should not be possible while start reading");
                    this._currentReads.push(createMutablePromise());
                    resolve(this._value);
                });
            });
            this._pendingTransactions.push({ promise: readTransaction, isRead: true });
            return toReturn;
            //}
        }
        finally {
            this._tryProcessNext();
        }
        // diag.throwCheck( this._currentWrite == null, "race, current write should not be possible while start reading" )
        // this._currentReads.push( createMutablePromise() )
        // return this._value
    }
    readEnd() {
        diag.throwCheck(this._currentReads.length > 0, "out of current reads, over decremented?");
        diag.throwCheck(this._currentWrite == null, "race, current write should not be possible while reading");
        this._currentReads.pop().resolve();
        this._tryProcessNext();
    }
    tryWriteBegin() {
        if (this._currentReads.length > 0 || this._currentWrite != null || this._pendingTransactions.length > 0) {
            return { lockAquired: false };
        }
        this._currentWrite = createMutablePromise();
        return { lockAquired: true, value: this._value };
    }
    async writeBegin() {
        const fastTry = this.tryWriteBegin();
        if (fastTry.lockAquired === true) {
            return fastTry.value;
        }
        try {
            const writeTransaction = createMutablePromise();
            const toReturn = new Promise((resolve, reject) => {
                void writeTransaction.then(() => {
                    diag.throwCheck(this._currentReads.length === 0, "should be no reads");
                    diag.throwCheck(this._currentWrite == null, "should be no other writes");
                    this._currentWrite = createMutablePromise();
                    resolve(this._value);
                });
            });
            this._pendingTransactions.push({ promise: writeTransaction, isRead: false });
            return toReturn;
        }
        finally {
            this._tryProcessNext();
        }
    }
    async writeEnd(value) {
        try {
            diag.throwCheck(this._currentWrite != null, "race, current write should be set");
            diag.throwCheck(this._currentReads.length === 0, "should be no reads");
            this._value = await Promise.resolve(value);
            const curWrite = this._currentWrite;
            this._currentWrite = undefined;
            curWrite.resolve();
        }
        finally {
            this._tryProcessNext();
        }
    }
}
exports.ReadWriteLock = ReadWriteLock;
class Autoscaler {
    constructor(options, backendWorker, 
    /** will be used to intercept failures (promise rejections) from the ```backendWorker``` function.  should return "FAIL" if it's a normal failure (to be returned to the caller) or "TOO_BUSY" if the request should be retried  */
    failureListener) {
        this.options = options;
        this.backendWorker = backendWorker;
        this.failureListener = failureListener;
        this.pendingCalls = [];
        this.activeCalls = [];
        /** submit a request to the backend worker.
         *
         * **Important note**: to avoid "unhandled promise rejections" you need to make sure the returned Promise has a catch() applied to it.
         * **NOT** just store the promise in an array to inspect later.  This is because if the request fails, the returned promise gets rejected, and if the Promise internal logic doesn't see a .catch() it will show the global "unhandled rejected promse" soft error message.
         */
        this.submitRequest = 
        //a worker with generic input/return args, cast to our specific worker function's sig
        (async (...args) => {
            const requesterPromise = createMutablePromise();
            this.pendingCalls.push({ args, requesterPromise });
            this._tryCallBackend();
            return requesterPromise;
        });
        this._lastTryCallTime = new Date();
        //apply defaults
        this.options = Object.assign({ busyExtraPenalty: 1 }, options);
        diag.throwCheck(this.options.minParallel >= 1, "minParallel needs to be 1 or more");
        this._metrics = { activeCount: 0, tooBusyWaitStart: new Date(0), lastGrow: new Date(0), lastMax: new Date(0), maxActive: options.minParallel, lastDecay: new Date(0), lastTooBusy: new Date(0) };
    }
    toJson() {
        return { pendingCalls: this.pendingCalls.length, activeCalls: this.activeCalls.length, metrics: this._metrics, options: this.options };
    }
    _tryCallBackend() {
        const now = new Date();
        try {
            while (true) {
                // ! /////////////  do housekeeping ///////////////////
                //check if we have to abort for various reasons
                if (this.pendingCalls.length === 0) {
                    //nothing to do
                    return;
                }
                if (this._metrics.activeCount >= this._metrics.maxActive) {
                    //make note that we are at our limit of requests
                    this._metrics.lastMax = now;
                }
                if (this.options.maxParallel != null && this._metrics.activeCount >= this.options.maxParallel) {
                    //at our hard limit of parallel requests
                    return;
                }
                if (this._metrics.activeCount >= this._metrics.maxActive //we are at our max...
                    && (this._metrics.lastGrow.getTime() + this.options.growDelayMs < now.getTime()) //we haven't grew recently...
                    && (this._metrics.tooBusyWaitStart.getTime() + this.options.busyGrowDelayMs < now.getTime()) //we are not in a options.busyWaitMs interval (haven't recieved a "TOO_BUSY" rejection recently...)
                ) {
                    //time to grow
                    this._metrics.maxActive++;
                    this._metrics.lastGrow = now;
                }
                const lastTryMsAgo = now.getTime() - this._lastTryCallTime.getTime();
                if (this.options.idleOrBusyDecreaseMs != null && this._metrics.lastDecay.getTime() + (this.options.idleOrBusyDecreaseMs + lastTryMsAgo) < now.getTime()) {
                    //havent decayed recently
                    if ((this._metrics.lastMax.getTime() + (this.options.idleOrBusyDecreaseMs + lastTryMsAgo) < now.getTime()) //havent been at max recently
                        || (this._metrics.lastTooBusy.getTime() + this.options.idleOrBusyDecreaseMs > now.getTime()) //OR we have gotten "TOO_BUSY" rejections since our last decay, so backoff
                    ) {
                        //time to reduce our maxActive
                        const reduceCount = 1 + Math.round((now.getTime() - this._metrics.lastMax.getTime()) / this.options.idleOrBusyDecreaseMs); //accumulating decays in case the autoScaler has been idle
                        diag.throwCheck(reduceCount >= 0, "active workers is <0.  should always be >=0");
                        this._metrics.maxActive = Math.max(this.options.minParallel, this._metrics.maxActive - reduceCount);
                        //pretend we are at max, to properly delay growing.
                        this._metrics.lastMax = now;
                        this._metrics.lastDecay = now;
                    }
                }
                if (this._metrics.activeCount >= this._metrics.maxActive) {
                    //we are at our maxActive, wait for a free slot
                    return;
                }
                // ! ////////// Done with housekeeping and didn't early abort.   time to call the backend  /////////////////
                diag.throwCheck(this.pendingCalls.length > 0, "race condition, expected pendingCalls length to be 1 or more");
                const { args, requesterPromise } = this.pendingCalls.shift();
                const activeMonitorPromise = Promise.resolve(this.backendWorker(...args))
                    .then((result) => {
                    requesterPromise.resolve(result);
                }, (err) => {
                    //failure, see what to do about it
                    const verdict = this.failureListener(err);
                    switch (verdict) {
                        case "TOO_BUSY":
                            {
                                const tooBusyNow = new Date();
                                this._metrics.lastTooBusy = tooBusyNow;
                                //apply special backoffPenaltyCount options, if they exist
                                if (this.options.busyExtraPenalty != null && this._metrics.tooBusyWaitStart.getTime() + this.options.busyGrowDelayMs < tooBusyNow.getTime()) {
                                    //this is a "fresh" backoff.
                                    //we have exceeded backend capacity and been notified with a "TOO_BUSY" failure.  reduce our maxParallel according to the options.backoffPenaltyCount
                                    this._metrics.maxActive = Math.max(this.options.minParallel, this._metrics.maxActive - this.options.busyExtraPenalty);
                                    //set our "fresh" tooBusy time
                                    this._metrics.tooBusyWaitStart = tooBusyNow;
                                }
                                //put request in front to try again
                                this.pendingCalls.unshift({ args, requesterPromise });
                            }
                            break;
                        case "FAIL":
                            requesterPromise.reject(err);
                            break;
                        default:
                            throw new exception.XlibException(`Autoscaler error.  failureListener() return value unexpected ("${verdict}").  must return "TOO_BUSY" or "FAIL".  `);
                    }
                    return Promise.resolve();
                })
                    .finally(() => {
                    //remove this from actives array
                    this._metrics.activeCount--;
                    _.remove(this.activeCalls, (activeCall) => activeCall.requesterPromise === requesterPromise);
                    //try another pass
                    this._tryCallBackend();
                });
                this._metrics.activeCount++;
                this.activeCalls.push({ args, requesterPromise, activeMonitorPromise });
            }
        }
        catch (_err) {
            //throw _err
            throw new exception.XlibException("Austoscaler error.  backend error occured.", { innerError: _err });
            //log.error( _err );
        }
        finally {
            this._lastTryCallTime = now;
            if (this.pendingCalls.length > 0 && this._metrics.activeCount >= this._metrics.maxActive) {
                //we have pending work, try again at minimum our next potential grow interval
                clearTimeout(this._heartbeatHandle); //only 1 pending callback, regardless of how many calls there were
                this._heartbeatHandle = setTimeout(() => { this._tryCallBackend(); }, this.options.growDelayMs);
            }
        }
    }
}
exports.Autoscaler = Autoscaler;
//# sourceMappingURL=promise.js.map