"use strict";
/* eslint-disable @typescript-eslint/ban-types */
/* eslint-disable @typescript-eslint/no-explicit-any */
/* eslint-disable dot-notation */
/** @packageDocumentation
 * launcher for mocha tests in the browser
 * @remarks
 * Two reasons for this:
 * 1) tests internally are coded to match jest style tests.  so this contains shims for converting the jest `it` function to be usable by mocha
 * 2) there isn't a standard way to launch mocha from webpack (the ```mocha-loader``` plugin is broken) so this loads mocha, then finds all test files and runs them
 *   - see: https://github.com/webpack-contrib/mocha-loader/issues/83
    */
try {
    //when testing, use source maps
    require("source-map-support/register").install();
}
catch (_a) { }
//bootstrap mocha, as per: https://mochajs.org/#running-mocha-in-the-browser
//import expect from "expect"
//if(globalThis.expect ==null)
// if ( typeof ( expect ) === "undefined" ) {
// 	window[ "expect" ] = require( "chai" )
// }
const mocha = require("mocha");
//import "mocha"
mocha.setup({
    ui: "bdd",
    timeout: 5000,
});
mocha.growl(); // enable web notification
/** Allows using Jest's ```it``` and ```describe``` definitions, even if using mocha.
 * only does this if the jest global is not present and the mocha global is
 * @packageDocumentation
 */
{
    const mochaIt = it;
    globalThis.it = ((name, fn, timeout) => {
        var _a;
        const _test = mochaIt(name, fn);
        if (timeout != null) {
            (_a = _test === null || _test === void 0 ? void 0 : _test.timeout) === null || _a === void 0 ? void 0 : _a.call(_test, timeout);
        }
        return _test;
    });
    if (globalThis.beforeAll == null) {
        globalThis.beforeAll = (fn, timeout) => {
            var _a;
            const mochaFn = before(fn);
            if (timeout != null) {
                (_a = mochaFn === null || mochaFn === void 0 ? void 0 : mochaFn.timeout) === null || _a === void 0 ? void 0 : _a.call(mochaFn, timeout);
            }
            return mochaFn;
        };
    }
}
//load up all test files following general advice from: https://stackoverflow.com/questions/32385219/mocha-tests-dont-run-with-webpack-and-mocha-loader
{
    const foundTestsRequire = require.context("..", true, /.*\.test\.js?$/);
    //console.log( `keys found = ${ foundTestsRequire.keys().length }` )
    for (const key of foundTestsRequire.keys()) {
        console.info(`MOCHA-TEST-LAUNCHER:  about to load tests for ${key}`);
        foundTestsRequire(key);
    }
}
//run tests
mocha.run();
//# sourceMappingURL=browser-mocha-test-launcher.js.map