import * as _ from "lodash";
export function arrayBufferToString(buffer) {
    return String.fromCharCode.apply(null, new Uint8Array(buffer));
}
export function stringToArrayBuffer(str) {
    const buf = new ArrayBuffer(str.length * 2); // 2 bytes for each char
    const bufView = new Uint16Array(buf);
    for (let i = 0, strLen = str.length; i < strLen; i++) {
        bufView[i] = str.charCodeAt(i);
    }
    return buf;
}
export function ipV4toInt(ip) {
    const parts = ip.split(".");
    let res = 0;
    res += parseInt(parts[0], 10) << 24;
    res += parseInt(parts[1], 10) << 16;
    res += parseInt(parts[2], 10) << 8;
    res += parseInt(parts[3], 10);
    return res;
}
export function intToIpV4(int) {
    const part1 = int & 255;
    const part2 = ((int >> 8) & 255);
    const part3 = ((int >> 16) & 255);
    const part4 = ((int >> 24) & 255);
    // tslint:disable-next-line: restrict-plus-operands
    return part4 + "." + part3 + "." + part2 + "." + part1;
}
import base64url from "base64url";
/**
 *  Converting to, and from, base64url https://en.wikipedia.org/wiki/Base64#RFC_4648
example:   base64=```'qL8R4QIcQ/ZsRqOAbeRfcZhilN/MksRtDaErMA=='``` base64Url=```'qL8R4QIcQ_ZsRqOAbeRfcZhilN_MksRtDaErMA'```
uses the npm package: https://www.npmjs.com/package/base64url
 */
export const base64Url = base64url;
//export const base64Url = _base64Url.default
/**
 *  base64 encode and decode functions
 */
// eslint-disable-next-line @typescript-eslint/no-namespace
export var base64;
(function (base64) {
    function encode(input) {
        const isStr = _.isString(input);
        if (isStr !== true || (typeof (btoa) === "undefined" && typeof (Buffer) !== "undefined")) {
            if (isStr) {
                //nodejs does not define bota or atob
                return Buffer.from(input, "utf8").toString("base64");
            }
            else {
                return input.toString("base64");
            }
        }
        return btoa(input);
    }
    base64.encode = encode;
    function decode(base64Encoded) {
        if (typeof (atob) === "undefined" && typeof (Buffer) !== "undefined") {
            //nodejs does not define bota or atob
            return Buffer.from(base64Encoded, "base64").toString("utf8");
        }
        return atob(base64Encoded);
    }
    base64.decode = decode;
    function toBuffer(base64Encoded) {
        return Buffer.from(base64Encoded, "base64");
    }
    base64.toBuffer = toBuffer;
})(base64 || (base64 = {}));
//# sourceMappingURL=convert.js.map