"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DebugRaceCheck = exports.RaceCheckException = exports.throwCheck = exports.Logger = void 0;
const exception = __importStar(require("./exception"));
const env = __importStar(require("./environment"));
const lodash_1 = __importDefault(require("lodash"));
const pino = __importStar(require("pino"));
class Logger {
    constructor(name, options = {}) {
        var _a;
        this.name = name;
        this.options = options;
        const envInfo = env.getEnvInfo();
        options = Object.assign({ ignore: "pid,hostname", translateTime: "yyyymmdd_HHMM:ss.l", level: envInfo.log, browserConsoleEnvLevel: ["dev"], browserConsoleLogLevel: ["trace", "debug"] }, options);
        this._pino = pino.default({
            name,
            level: (_a = options.level) === null || _a === void 0 ? void 0 : _a.toString().toLowerCase(),
            //options here: https://github.com/pinojs/pino-pretty
            prettyPrint: (() => {
                if (lodash_1.default.includes(["uat", "prod"], envInfo.env)) { //}envInfo.env  >= env.EnvLevel.UAT ) {
                    return false;
                }
                return {
                    //colorize: chalk.supportsColor as boolean, // --colorize  already the default so can disable this option
                    //crlf: false, // --crlf
                    errorLikeObjectKeys: ["err"],
                    //errorProps: "type,message", // --errorProps
                    // levelFirst: false, // --levelFirst
                    // messageKey: "msg", // --messageKey
                    // levelKey: "level", // --levelKey
                    // messageFormat: false, // --messageFormat
                    //messageFormat: "{level} -------",
                    //timestampKey: "time", // --timestampKey
                    translateTime: options.translateTime,
                    // search: "foo == `bar`", // --search
                    ignore: options.ignore,
                };
            })(),
        });
        if (envInfo.platform === "browser" && lodash_1.default.includes(options.browserConsoleEnvLevel, envInfo.env) && lodash_1.default.includes(options.browserConsoleLogLevel, envInfo.log)) {
            //method replacement for sourcelinks, idea from: https://stackoverflow.com/questions/9559725/extending-console-log-without-affecting-log-line
            this.trace = Function.bind.call(console.trace, console);
            this.debug = Function.bind.call(console.debug, console);
            this.info = Function.bind.call(console.log, console);
            this.warn = Function.bind.call(console.warn, console);
            this.error = Function.bind.call(console.error, console);
            this.fatal = Function.bind.call(console.error, console);
            this.silent = () => { };
        }
    }
    _callPino(method, obj, msg) {
        this._pino[method](obj, msg);
    }
    trace(obj, msg) {
        this._callPino("trace", obj, msg);
    }
    debug(obj, msg) {
        this._callPino("debug", obj, msg);
    }
    info(obj, msg) {
        this._callPino("info", obj, msg);
    }
    warn(obj, msg) {
        this._callPino("warn", obj, msg);
    }
    error(obj, msg) {
        this._callPino("error", obj, msg);
    }
    fatal(obj, msg) {
        this._callPino("fatal", obj, msg);
    }
    silent(obj, msg) {
        this._callPino("silent", obj, msg);
    }
    assert(condition, obj, msg) {
        if (condition === true) {
            return;
        }
        this.error(obj, msg);
        throw new exception.Exception("assert failed", { details: Object.assign({ msg }, obj) });
    }
}
exports.Logger = Logger;
// interface LogFn {
// 	( msg: string ): void;
// 	( obj: object, msg?: string ): void;
// }
/** @internal helper to throw a XlibException if the condition !==true */
function throwCheck(condition, errorMsg) {
    errorMsg = errorMsg !== null && errorMsg !== void 0 ? errorMsg : "xlib.diagnostics.throwCheck(false) input condition was false.  Inspect callsite (see error's callstack)";
    if (condition !== true) {
        throw new exception.XlibException(errorMsg, { stackFramesToTruncate: 1 });
    }
}
exports.throwCheck = throwCheck;
/** thrown on race-check failures */
class RaceCheckException extends exception.XlibException {
}
exports.RaceCheckException = RaceCheckException;
/** DEBUG ONLY!  noops self if environment.logLevel is greater than DEBUG helper to check for algorithm race conditions (example: removing from a collection while enumerating) */
class DebugRaceCheck {
    constructor(debugText = "") {
        this.debugText = debugText;
        this._version = 0;
        this._lockVersion = 0;
        this._isDisabled = false;
        if (!env.isDebug()) {
            //noop racecheck functions
            this.edit = () => { };
            this.enter = () => { };
            this.exit = () => { };
            this.poke = () => { };
            this._isDisabled = true;
        }
    }
    enter() {
        if (this._lockVersion !== 0) {
            //set lock version to -1 to cause the other thread caller to assert also
            this._lockVersion = -1;
            throw new RaceCheckException("DebugRaceCheck.Enter() failed!  Something else has this locked!  Additional Info=" + String(this.debugText));
        }
        this._lockVersion = this._version;
        this._version++;
    }
    exit() {
        if (this._lockVersion !== this._version - 1) {
            if (this._lockVersion === 0) {
                throw new RaceCheckException("DebugRaceCheck.Exit() failed!  Already unlocked and we are calling unlock again!  Additional Info=" + String(this.debugText));
            }
            else {
                throw new RaceCheckException("DebugRaceCheck.Exit() failed!  internal state is corrupted, moste likely from multithreading  Additional Info=" + String(this.debugText));
            }
            //cause the assert to keep occuring if this failed
            //return;
        }
        this._lockVersion = 0;
    }
    /** quickly locks and unlocks */
    edit() {
        this.enter();
        this.exit();
    }
    /** ensures that this raceCheck is not currently locked */
    poke() {
        this._version++;
        if (this._lockVersion !== 0) {
            throw new RaceCheckException("DebugRaceCheck.Exit() failed!internal state is corrupted, moste likely from multithreading  Additional Info=" + String(this.debugText));
        }
    }
    toString() {
        if (this._isDisabled === false) {
            const isEdit = this._lockVersion === 0 ? false : true;
            return "version=" + String(this._version) + " isBeingEdited=" + String(isEdit);
        }
        else {
            return "DebugRaceCheck is DISABLED when baselib.environment.logLevel > EnvLevel.DEBUG.  use baselib.concurrency.Lock if you need something in production";
        }
    }
}
exports.DebugRaceCheck = DebugRaceCheck;
// if ( !environment.isDebug() ) {
// 	//noop racecheck functions
// 	// tslint:disable-next-line: no-empty
// 	DebugRaceCheck.prototype.edit = () => { };
// 	// tslint:disable-next-line: no-empty
// 	DebugRaceCheck.prototype.enter = () => { };
// 	// tslint:disable-next-line: no-empty
// 	DebugRaceCheck.prototype.exit = () => { };
// 	// tslint:disable-next-line: no-empty
// 	DebugRaceCheck.prototype.poke = () => { };
// }
//# sourceMappingURL=diagnostics.js.map