"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.errorToJson = exports.errorToString = exports.toError = exports.HttpStatusCodeException = exports.XlibException = exports.Exception = exports.getStackTrace = void 0;
const _ = __importStar(require("lodash"));
function getStackTrace() {
    const stackStr = Error().stack;
    if (stackStr == null) {
        return [];
    }
    return stackStr.split("\n").slice(2); //ln 0 is "Error", ln 1 is self.
}
exports.getStackTrace = getStackTrace;
/** An `Error` object that includes improved features
 * @remarks
 * ## important usage
 * If you derive from this and have a custom constructor, be sure to call the base class's `super()` in your constructor.  This will ensure that your derived class inherits from `Error`
 *
 * @example
 * ```typescript
 * class MyException extends Exception{ } throw new MyException("boo");
 * ```
 *
 * @see
core idea from https://www.typescriptlang.org/docs/handbook/2/classes.html#inheriting-built-in-types

*/ //Note: you can control if additional properties are logged via options.logProperties=true.  While stack frames will only be logged when envLevel!=PROD.
class Exception extends Error {
    constructor(message, options = {}) {
        var _a, _b;
        super(message);
        this.message = message;
        Object.setPrototypeOf(this, new.target.prototype); //fix inheritance, new in ts2.2: https://www.typescriptlang.org/docs/handbook/release-notes/typescript-2-2.html
        //jsHelper.setPrototypeOf( this, new.target.prototype );
        options = Object.assign({ stackFramesToTruncate: 0 }, options);
        //this.options = options;
        //this.data = options.any;
        if (options.innerError != null) {
            //make sure that what's passed is actually an error object
            options.innerError = toError(options.innerError);
        }
        this.details = options.details;
        //if (environment.logLevel > environment.LogLevel.DEBUG) {
        //	innerException = null;
        //} else {
        //	if (innerException != null) {
        //		this.message = message + ": " + innerException.message;
        //	}
        //}
        /** split up the stack for manipulation during ```.ctor()```.  will recombine at end of  ```.ctor()```. */
        let splitStack = (_b = (_a = super.stack) === null || _a === void 0 ? void 0 : _a.split("\n")) !== null && _b !== void 0 ? _b : [];
        if (options.innerError != null && typeof (options.innerError.stack) === "string") {
            const newStack = options.innerError.stack.split("\n");
            newStack.unshift("innerException stack:");
            newStack.unshift(splitStack[1]);
            newStack.unshift(splitStack[0]);
            splitStack = newStack;
        }
        //truncate stackframes
        if (options.stackFramesToTruncate > 0) {
            //in nodejs, the first frame is actually message, so keep it.
            const messageFrame = splitStack.shift();
            for (let i = 0; i < options.stackFramesToTruncate; i++) {
                splitStack.shift();
            }
            if (messageFrame != null) {
                //put message back
                splitStack.unshift(messageFrame);
            }
        }
        //max stackframes
        if (options.maxStackFrames != null) {
            splitStack.length = options.maxStackFrames + 1;
        }
        //recombine our array back to the normal stack string.
        this.stack = splitStack.join("\n");
        if (options.innerError != null && typeof (options.innerError.message) === "string" && message.includes(options.innerError.message) !== true) {
            //include the innerError message in our message, but only if it wasn't manually added there already.
            this.message = message + "   innerException: " + options.innerError.message;
        }
        else {
            this.message = message; //making sure it's set for explicit order when serializing to JSON
        }
        //get name based on type.  snippet taken from ./runtime/reflection.ts code
        if ((this.constructor).name != null) {
            //es6
            this.name = (this.constructor).name;
        }
        else {
            //es5
            const results = (Exception._getTypeNameOrFuncNameRegex).exec((this).constructor.toString());
            this.name = (results != null && results.length > 1) ? results[1] : "";
        }
        this.innerError = options.innerError; //putting this last to help ensure json serialization order
    }
    /** includes stack track in string*/
    toString() {
        return errorToString(this);
    }
    toJson() {
        return errorToJson(this);
    }
}
exports.Exception = Exception;
//public options: IExceptionOptions;
Exception._getTypeNameOrFuncNameRegex = /function (.{1,})\(/;
/** all errors thrown by xlib extend this error type */
class XlibException extends Exception {
}
exports.XlibException = XlibException;
/**
 * an exception that includes a statusCode for returning via http requests
 */
class HttpStatusCodeException extends Exception {
    constructor(message, statusCode, options) {
        super(message, options);
        this.statusCode = statusCode;
    }
}
exports.HttpStatusCodeException = HttpStatusCodeException;
/** ensures you have an Error object.
    * If you pass in an error, you will get it back.  If you pass in anything else, a new Error will be created and a string represtation of your input will be set as the ```message``` parameter.
 @remarks
    * useful for ```try{}catch(ex){}``` statements, where you are uncertain what the thrown error is.
    * @example
 ```typescript
    * try{
    *  await someClass.someMethod();
    * }catch(_err){
    *  const err = __.diag.toError(_err);
    * //use err as a normal Error object
    * }
 ```
*/
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function toError(ex) {
    return toError_worker(ex, new Set());
}
exports.toError = toError;
function toError_worker(ex, /** prevent infinite recursion */ _innerErrorRecursionTracker) {
    if (_innerErrorRecursionTracker.has(ex)) {
        return new Error("circular reference in innerError");
    }
    _innerErrorRecursionTracker.add(ex);
    if (ex == null) {
        throw new XlibException("missing argument 'ex'");
    }
    if (ex instanceof Error) {
        return ex;
    }
    let message;
    if (typeof ex === "string") {
        message = ex;
    }
    else if (typeof ex.message === "string") {
        message = ex.message;
    }
    else {
        try {
            const json = JSON.parse(ex);
            message = JSON.stringify(json).slice(0, 200);
            //message = stringHelper.summarize( JSON.stringify( json ), 200 );
        }
        catch (_a) {
            message = "??? unknown, can not parse ???";
        }
    }
    const toReturn = new Exception(message, { maxStackFrames: 0 });
    //apply other properties
    // if ( typeof ( ex.innerError ) === "string" ) {
    // 	toReturn.innerError = toError(ex.innerError)
    // }
    if (ex.innerError != null) {
        toReturn.innerError = toError_worker(ex.innerError, _innerErrorRecursionTracker);
    }
    toReturn.details = ex.details;
    if (typeof (ex.name) === "string") {
        toReturn.name = ex.name;
    }
    if (ex.stack != null) {
        if (typeof (ex.stack) === "string") {
            toReturn.stack = ex.stack;
        }
        else if (Array.isArray(ex.stack) && ex.stack.join != null) {
            toReturn.stack = ex.stack.join(",\n");
        }
    }
    return toReturn;
}
/** get a string representation of the error */
function errorToString(ex, options) {
    const exJson = errorToJson(ex, options);
    //exJson.stack = exJson.stack?.join( "\n" ) ?? ""
    return JSON.stringify(exJson);
}
exports.errorToString = errorToString;
const environment = __importStar(require("../environment"));
// let t1: ErrorAsJson<MyException>;
/** convert an error and all it's properties to JSON.   */
function errorToJson(error, options = {}) {
    return errorToJson_worker(error, options, new Set());
}
exports.errorToJson = errorToJson;
function errorToJson_worker(error, options = {}, _circularRefDetection) {
    var _a, _b, _c;
    if (_circularRefDetection.has(error)) {
        return {
            name: "circular reference",
            message: "error converting innerError to json due to circular reference",
        };
    }
    _circularRefDetection.add(error);
    options = Object.assign({}, options);
    if (error == null) {
        //return { message: "", name: "NullNotError" } as any
    }
    //options = { ...options }
    //let error = toError( _error );
    let stackArray;
    const innerError = error.innerError;
    //convert stack to array
    let stack = error.stack;
    if (stack == null) {
        stack = "none";
    }
    if (typeof stack === "string") {
        //toReturn.stack = toReturn.stack.substring(0,400);
        stackArray = stack.split("\n");
    }
    else if (_.isArray(stack)) {
        //array;
        stackArray = stack;
    }
    else if (typeof stack.toString === "function") {
        stackArray = [stack.toString()];
    }
    else {
        stackArray = ["unknown"];
    }
    //sanitize: remove stack traces if in production
    if (options.alwaysShowFullStack !== true && environment.isProd() && environment.isDebug() === false) {
        switch (environment.getLogLevel()) {
            case "info":
                options.maxStacks = Math.min((_a = options.maxStacks) !== null && _a !== void 0 ? _a : 2, 2);
                stackArray[1] = "only 1 stackFrame is shown because env=PROD && log=INFO";
                break;
            default:
                options.maxStacks = Math.min((_b = options.maxStacks) !== null && _b !== void 0 ? _b : 1, 1);
                stackArray[0] = "no stackFrames shown because env=PROD && log>INFO";
        }
    }
    // if ( options.alwaysShowFullStack !== true && environment.isProd() && environment.getLogLevel() > environment.LogLevel.TRACE.logLevel > environment.LogLevel.DEBUG && environment.envLevel > environment.EnvLevel.TEST ) {
    // 	//sanitize
    // 	stackArray = [ "no stack or extra properties unless envLevel=DEV|TEST, or logLevel=DEBUG|TRACE" ]
    // 	innerError = undefined
    // } else {
    // }
    //sanitize: set max stack length
    stackArray.slice(0, (_c = options.maxStacks) !== null && _c !== void 0 ? _c : Number.MAX_SAFE_INTEGER);
    // if ( options.maxStacks != null && stackArray.length > options.maxStacks ) {
    // 	stackArray.length = options.maxStacks
    // }
    /////////////////////////////
    let serialized;
    if (options.alwaysShowProperties === true) {
        serialized = _.cloneDeep(error);
    }
    else {
        serialized = {
            name: error.name,
            message: error.message,
        };
    }
    //common values
    serialized.stack = stackArray;
    if (innerError != null) {
        serialized.innerError = errorToJson_worker(innerError, options, _circularRefDetection);
    }
    return serialized;
    // try {
    // 	if ( options.alwaysShowProperties !== true && environment.logLevel > environment.LogLevel.DEBUG && environment.envLevel > environment.EnvLevel.TEST ) {
    // 		//don't show extra properties
    // 		const _tmpEx = error as Exception
    // 		serialized = {
    // 			name: _tmpEx.name,
    // 			message: _tmpEx.message,
    // 			innerError: _tmpEx.innerError,
    // 			stack: _tmpEx.stack
    // 		}
    // 		serialized = JSON.parse( JSON.stringify( serialized ) )
    // 	} else {
    // 		serialized = JSON.parse( JSON.stringify( error ) )
    // 	}
    // } catch ( ex ) {
    // 	serialized = {} as any
    // }
    // if ( options.maxStacks != null && stackArray.length > options.maxStacks ) {
    // 	stackArray.length = options.maxStacks
    // }
    // let innerErrorJson: IErrorJson | undefined
    // if ( innerError != null ) {
    // 	try {
    // 		innerErrorJson = errorToJson( innerError )
    // 	} catch {
    // 		//eat error
    // 		//innerErrorJson = serialized.innerError as any;
    // 		innerErrorJson = { name: "UnknownErrorToJson", message: "" }
    // 	}
    // }
    // const toReturn = {
    // 	...serialized,
    // 	name: error.name,
    // 	message: error.message,
    // 	stack: stackArray,
    // 	innerError: innerErrorJson,
    // }
    // return toReturn as any
}
//# sourceMappingURL=exception.js.map