"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseEnum = void 0;
/**
 * parse a string or num into an enum value.  Designed for "standard" enums that map to numerical values.  not for string enums.
 * @remarks
 * for string enums, just do a keys lookup
 */
function parseEnum(enumClass, key, { caseSensitive = false } = {}) {
    //definition hint (return type) from :https://github.com/Microsoft/TypeScript/issues/18869#issuecomment-333339975
    //IEnum details from: https://github.com/microsoft/TypeScript/issues/30611#issuecomment-479087883
    if (key == null)
        return undefined;
    const result = enumClass[key];
    if (typeof result === "number") {
        //numbers are only ever values, so just return that
        return result;
    }
    if (typeof key === "number" || !isNaN(Number.parseInt(key))) {
        //ensure this is a proper value found in the enum, then return the value itself
        if (result != null)
            return key;
        //if numeric key doesn't match, can stop here
        return undefined;
    }
    //need to itterate all keys trying to find a match	
    if (caseSensitive === true) {
        for (const k in enumClass) {
            if (k.toString() === key)
                return enumClass[k];
        }
    }
    else {
        const keyInsensitive = key.toString().toLowerCase();
        for (const k in enumClass) {
            if (k.toString().toLowerCase() === keyInsensitive) {
                if (typeof k === "number")
                    return k;
                return enumClass[k];
            }
        }
    }
    return undefined;
}
exports.parseEnum = parseEnum;
//# sourceMappingURL=enums.js.map