"use strict";
/* tslint:disable:no-bitwise */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseBoolean = exports.parseFloat = exports.parseInt = exports.toStringDigitGroupings = exports.toInt = exports.Interpolator = exports.clamp = exports.ClampType = exports.countDigits = exports.aboutEqual = exports.round = exports.isReal = exports.hash = exports.format = exports.INT_MIN = exports.INT_MAX = exports.INT32_MIN = exports.INT32_MAX = exports.INT16_MIN = exports.INT16_MAX = exports.INT8_MIN = exports.INT8_MAX = void 0;
const exception = __importStar(require("./exception"));
//import * as crypto from "crypto"
// import crypto = require( "crypto" );
/** max for a signed 8bit integer. (127) */
exports.INT8_MAX = 127;
/** min for a signed 8bit integer. (-128) */
exports.INT8_MIN = -128;
/** max for a signed 16bit integer. (32767) */
exports.INT16_MAX = 32767;
exports.INT16_MIN = -32768;
exports.INT32_MAX = 2147483647;
exports.INT32_MIN = -2147483648;
/** largest exact integer supported in javascript.  2^53 -1.  (53 bit mantissa) after this, floating point rounding will occur
from http://stackoverflow.com/questions/307179/what-is-javascripts-max-int-whats-the-highest-integer-value-a-number-can-go-t */
exports.INT_MAX = 9007199254740991;
/** largest exact integer supported in javascript.  2^53 -1.  (53 bit mantissa) after this, floating point rounding will occur
from http://stackoverflow.com/questions/307179/what-is-javascripts-max-int-whats-the-highest-integer-value-a-number-can-go-t */
exports.INT_MIN = -9007199254740991;
/**
 * formats values into strings using special heuristics to guess what is user friendly.
 * @param num
 */
function format(value, /** default=5 */ significantDigits = 5, separatorChar) {
    if (isNaN(value)) {
        return "NaN";
    }
    const digits = countDigits(value);
    const roundDigits = digits - significantDigits;
    value = round(value, roundDigits);
    return toStringDigitGroupings(value, separatorChar);
}
exports.format = format;
/** fast hash a int32, not very great spread */
function hash(input) {
    //from http://stackoverflow.com/questions/9624963/java-simplest-integer-hash
    if (!(input <= exports.INT32_MAX && input % 1 === 0)) {
        throw new exception.XlibException("must supply integer");
    }
    input ^= (input >>> 20) ^ (input >>> 12);
    return input ^ (input >>> 7) ^ (input >>> 4);
}
exports.hash = hash;
/**
 * ensure that the value specified is a number and is finite.
 * @param x
 */
function isReal(x) {
    if (isNaN(x) || !isFinite(x)) {
        return false;
    }
    return true;
}
exports.isReal = isReal;
function round(value, 
/** default = 0.  round(123.456,10)=>0;    round(123.456,2)=>100;   round(123.456,-2)=>123.46;   round(123.456,0)=>123;   round(123.456,-10)=>123.456;
*/
digitsOrNegativeForDecimals = 0) {
    const mult = Math.pow(10, -digitsOrNegativeForDecimals);
    return Math.round(value * mult) / mult;
}
exports.round = round;
/** check that two nombers are close enough to eachother.     all parameters are combined when calculating, IE:   ```maxValue =  ( ( input * ( 1 + percentTollerance ) )  + spreadTollerance ) ```*/
function aboutEqual(input, checkAgainst, 
/** how many percent different the two numbers can be
 *
 * Default is ```0```, meaning this parameter is ignored */
percentTollerance, 
/** a fixed spread tollerance that the numbers should be within
 *
 * Default is ```0```, meaning this parameter is ignored */
spreadTollerance = 0) {
    return (((input * (1 + percentTollerance)) + spreadTollerance) >= checkAgainst
        && ((input * (1 - percentTollerance)) - spreadTollerance) <= checkAgainst);
}
exports.aboutEqual = aboutEqual;
/**
 *  count number of digits in a number
 * @param value
 * @param countDecimals
 */
function countDigits(value, /** default false, count only the whole digits */ countDecimals = false) {
    //solution from http://stackoverflow.com/questions/14879691/get-number-of-digits-with-javascript
    if (countDecimals === true) {
        value = Number(String(value).replace(/[^0-9]/g, ""));
    }
    return Math.max(Math.floor(Math.log(Math.abs(value)) * Math.LOG10E), 0) + 1;
}
exports.countDigits = countDigits;
/** how the clamp function should process the input (what kind of result should be returned) */
var ClampType;
(function (ClampType) {
    /** value not set, an error */
    ClampType[ClampType["none"] = 0] = "none";
    /** if value is below min, returns min.  likewise for above max */
    ClampType[ClampType["clamp"] = 1] = "clamp";
    /** if value is below min, loops over to max (or vice-versa) */
    ClampType[ClampType["loop"] = 2] = "loop";
    /** oscillate the returned value between min/max range */
    ClampType[ClampType["bounce"] = 3] = "bounce";
    /** return the remainder (amount value lies beyond min/max range), not the clamped value.
    if inside clamp range, returns zero.  if below min, returns negative.  if above range, returns positive. */
    ClampType[ClampType["remainder"] = 4] = "remainder";
})(ClampType = exports.ClampType || (exports.ClampType = {}));
/**
 *  clamp a number to be within the range specified
 */
function clamp(value, min_inc, max_inc, /** defaults to "clamp" */ clampType = ClampType.clamp) {
    const range = max_inc - min_inc;
    let remainder;
    if (value < min_inc) {
        remainder = value - min_inc;
    }
    else if (value > max_inc) {
        remainder = value - max_inc;
    }
    else {
        remainder = 0;
    }
    if (remainder === 0) {
        if (clampType === ClampType.remainder) {
            return 0;
        }
        return value;
    }
    const modRemainder = remainder % range;
    switch (clampType) {
        case ClampType.clamp:
            if (remainder < 0) {
                return min_inc;
            }
            return max_inc;
        case ClampType.remainder:
            return remainder;
        case ClampType.loop:
            if (remainder < 0) {
                return max_inc - modRemainder;
            }
            return min_inc + modRemainder;
        case ClampType.bounce:
            {
                const flipBounce = (Math.floor(Math.abs(remainder / range)) % 2) === 1;
                if ((remainder < 0 && !flipBounce) || (remainder > 0 && flipBounce)) {
                    return min_inc + modRemainder;
                }
                return max_inc - modRemainder;
            }
        default:
            throw new exception.XlibException("unknown ClampType: " + ClampType[clampType]);
    }
}
exports.clamp = clamp;
/** interpolate between values
base implementation works for numbers.  override .update() to extend functionality to other types */
class Interpolator {
    constructor(start, min, max, rate, isBounce, isEnabled = true) {
        this.start = start;
        this.min = min;
        this.max = max;
        this.rate = rate;
        this.isBounce = isBounce;
        this.isEnabled = isEnabled;
        this.clampType = isBounce ? ClampType.bounce : ClampType.loop;
        this.current = start;
        this.startRate = rate;
    }
    update(elapsedMs) {
        if (!this.isEnabled) {
            return this.current;
        }
        const delta = this.rate * elapsedMs / 1000;
        const newCurrent = this.current + delta;
        this.current = clamp(newCurrent, this.min, this.max, this.clampType);
        if (this.isBounce) {
            //if we do a bounce, flip our rate (this is buggy if we are going so fast to do multiple bounces in 1 update, but that"s acceptable for now)
            const remainder = clamp(newCurrent, this.min, this.max, ClampType.remainder);
            if (remainder !== 0) {
                this.rate = -this.rate;
            }
        }
        return this.current;
    }
}
exports.Interpolator = Interpolator;
/** truncate a float to int.   negative number safe, and fast performance */
function toInt(value) {
    //using bitwise operator, as per: http://stackoverflow.com/questions/596467/how-do-i-convert-a-float-number-to-a-whole-number-in-javascript
    return ~~value;
}
exports.toInt = toInt;
/**
 * a string with a separator for every number digit group (more than 4 digits)
 * @param num
 */
function toStringDigitGroupings(num, 
/** default comma */
separatorChar = ",", 
/** if true, decimals will have digits grouped with a space.  default=false */
groupDecimalsWithSpace = false) {
    if (isNaN(num)) {
        return "NaN";
    }
    const str = num.toString().split(".");
    if (str[0].length >= 5) {
        str[0] = str[0].replace(/(\d)(?=(\d{3})+$)/g, "$1" + separatorChar);
    }
    if (groupDecimalsWithSpace === true) {
        if (str[1] != null && str[1].length >= 5) {
            str[1] = str[1].replace(/(\d{3})/g, "$1 ");
        }
    }
    return str.join(".");
}
exports.toStringDigitGroupings = toStringDigitGroupings;
/** parses the value into an integer. */
function parseInt(toParse, /** invalid strings are returned as this (default=NaN) */ invalidResult = NaN, radix = 10) {
    //logger.assert(typeof (toParse) === "string", "input is not a string");
    const result = global.parseInt(toParse, radix);
    if (isNaN(result)) {
        return invalidResult;
    }
    else {
        return result;
    }
}
exports.parseInt = parseInt;
/** parses the value into a float. */
function parseFloat(toParse, /** invalid strings are returned as this (default=NaN) */ invalidResult = NaN, /** the default parseFloat implementation allows for trailing text.  specifying isStrict=TRUE makes only numbers + "Infinity" allowed to be parsed.   */ isStrict = false) {
    let result;
    if (isStrict === true) {
        if (/^(-|\+)?([0-9]+(\.[0-9]+)?|Infinity)$/
            .test(toParse)) {
            result = Number(toParse);
        }
        else {
            result = NaN;
        }
    }
    else {
        result = global.parseFloat(toParse);
    }
    if (isNaN(result)) {
        return invalidResult;
    }
    else {
        return result;
    }
}
exports.parseFloat = parseFloat;
function parseBoolean(toParse, invalidResult = false, throwOnInvalid = false) {
    if (typeof toParse === "string") {
        const str = toParse.toLowerCase().trim();
        switch (str) {
            case "true":
                //case "yes":
                //case "1":
                return true;
            case "false":
                //case "no":
                //case "0":
                return false;
            default:
                if (throwOnInvalid === true) {
                    throw new exception.XlibException("unable to parseBoolean on input value= " + toParse);
                }
                return invalidResult;
        }
    }
    if (typeof toParse === "number") {
        return toParse !== 0;
    }
    if (throwOnInvalid === true) {
        throw new exception.XlibException(`unable to parseBoolean on input value= ${toParse}`);
    }
    return invalidResult;
}
exports.parseBoolean = parseBoolean;
//module tests {
//	"use strict";
//	describe("abc", () => {
//		describe("second level", () => {
//			it("should fail", () => {
//				throw new Error("boom!");
//			});
//		});
//	});
//}
//# sourceMappingURL=num.js.map