import crypto from "isomorphic-webcrypto";
export { crypto };
import * as jwt from "jsonwebtoken";
export { jwt };
/** supported hash algo names. */
export declare type HASH_ALGO_NAMES = "SHA-256" | "SHA-384" | "SHA-512";
/**
 * defaults to generate a sha512 hash of your inputs, and returns it as a base64 encoded string. (88 characters in length)
 * optional to use a different algo
 * @param input
 */
export declare function hash(msg: string, algoName?: HASH_ALGO_NAMES): Promise<string>;
/**
 * returns a user-friendly alpha-numeric key.  By Default, constructs a cryptographically secure random one, or parses and normalizes one provided by userInput.
 * The key comprises the characters 0-9 and a-z, but not the characters o,i,l,u to avoid human transcription errors.
 * When parsing userInput, non-alphanumeric characters are discarded, and if the characters o,i,l,u are used, they are converted to 0,1,1,v respectively.
 * @param digits default = 25. length of the key you want to return, NOT including the digitGrouping seperators.
 * @param digitGroupings default=5.  to aid in humanreadableness, adds a seperator between groups of digits this size.
 * @param userInputToParse optional, attempts to parse user input, outputs a normalized key that can be used for comparison against the stored userkey.
 * @param groupingSeperator set the character used for sperating digit groups.  default is "-"
 * @param suppressThrowOnBadInput default false.  if true, returns empty string ("") when a blatently invalid key is detected.  when false, thrown an exception.
 */
export declare function humanFriendlyKey(digits?: number, digitGroupings?: number, userInputToParse?: string, groupingSeperator?: string, suppressThrowOnBadInput?: boolean): string;
/** well known elliptic curves.   note:
* see: https://w3c.github.io/webcrypto/#ecdsa
 */
export declare type ECNamedCurves = "P-256" | "P-384" | "P-521";
export declare type AsymAlgoName = ECNamedCurves;
export declare function genECKey(namedCurve: ECNamedCurves): Promise<CryptoKeyPair>;
/*** export the key pair in pkcs#8 PEM format */
export declare function exportKeyPkcs8(cryptoKeyPair: CryptoKeyPair | PromiseLike<CryptoKeyPair>): Promise<{
    pub: string;
    pri: string;
}>;
//# sourceMappingURL=security.d.ts.map